<?php

# composer require aliyuncs/oss-sdk-php jaeger/querylist tightenco/collect "5.*"
require __DIR__ . '/vendor/autoload.php';

use OSS\OssClient;
use QL\QueryList;
use Tightenco\Collect\Support\Collection;

ini_set('date.timezone', 'Asia/Shanghai');
libxml_use_internal_errors(true);

// 获取存储桶名称
$bucket = getenv('oss_bucket');
// 配置文件名
$config_name = 'config.json';
// 配置全局存储
$config = [];
// 推送全局存储
$push = [];
// OSS 全局存储
$oss = null;

function initializer($context) {
    global $config, $oss;

    if(is_null($oss)) {
        // 实例化 OSS
        // $oss = new OssClient(
        //     'yourAccessKeyId',
        //     'yourAccessKeySecret',
        //     'https://' . getenv('oss_endpoint')
        // );
        $oss = new OssClient(
            $context['credentials']['accessKeyId'],
            $context['credentials']['accessKeySecret'],
            'https://' . getenv('oss_endpoint'),
            false,
            $context['credentials']['securityToken']
        );
    }

    $config = get_config();
}

function handler($event, $context) {
    global $config, $push;

    // 复制一份配置数据，最后判断好判断是否需要更新 OSS 上面的配置文件。
    $_config = $config;

    // 清空推送
    $push = [];

    // 获取需要监控的厂商
    $providers = explode(',', getenv('providers') ?? 'aliyun,tencent,huawei');

    array_map(function($name) use(&$push) {
        if(function_exists('get_' . $name)) {
            call_user_func('get_' . $name);
        }
    }, $providers);

    // 进行推送
    foreach($push as $item) {
        send(...$item);
    }

    if($config != $_config) {
        save_config($config);
    }

    return 1;
}

/**
 * 获取公告列表
 * 
 * @param string  $provider  厂商
 * @return array
 */
function get_list($provider) {
    global $config;

    // 获取厂商公告列表数据
    $data = call_user_func('get_' . $provider . '_list');

    // 判断配置数据里面有没有该厂商，没有的话保存进去然后返回。
    if(!isset($config[$provider])) {
        $config[$provider] = $data;

        return [];
    }

    // 对当前读取的公告数据和配置数据里面的进行比对拿到不存在的。
    $list = collect(normalize_list($data))
                ->diffKeys(normalize_list($config[$provider]))
                ->all();

    return $list ? array_reverse($list) : [];
}

/**
 * 获取阿里云
 * 
 * @return array
 */
function get_aliyun() {
    $provider = 'aliyun';

    $list = get_list($provider);

    if(!$list) {
        return [];
    }

    return save_push($provider, '阿里云', '#se-knowledge', $list);
}

/**
 * 获取阿里云公告列表
 * 
 * @return array
 */
function get_aliyun_list() {
    return  QueryList::get('https://help.aliyun.com/noticelist.html')
                ->rules([
                    'name' => ['a', 'text'],
                    'href' => ['a', 'href'],
                ])
                ->range('.help-notice-box ul > li')
                ->query()
                ->queryData(function($item) {
                    $item['href'] = 'https://help.aliyun.com' . $item['href'];
                    return $item;
                });
}

/**
 * 获取腾讯云
 * 
 * @return array
 */
function get_tencent() {
    $provider = 'tencent';

    $list = get_list($provider);

    if(!$list) {
        return [];
    }

    return save_push($provider, '腾讯云', '.msg-d-bd', $list);
}

/**
 * 获取腾讯云公告列表
 * 
 * @return array
 */
function get_tencent_list() {
    // api 接口地址
    $api_url = 'https://cloud.tencent.com/announce/ajax?action=getAnnounceList';

    $data = json_decode(file_get_contents($api_url), true)['data']['rows'];
    
    return array_map(function($item) {
        return [
            'name' => $item['title'],
            'href' => 'https://cloud.tencent.com/announce/detail/' . $item['announceId'],
        ];
    }, $data);
}

/**
 * 获取华为云
 * 
 * @return array
 */
function get_huawei() {
    $provider = 'huawei';

    $list = get_list($provider);

    if(!$list) {
        return [];
    }

    return save_push($provider, '华为云', '.notice-content', $list);
}

/**
 * 获取华为云公告列表
 * 
 * @return array
 */
function get_huawei_list() {
    return  QueryList::get('https://www.huaweicloud.com/notice.html')
                ->rules([
                    'name' => ['.content > a', 'bi_name'],
                    'href' => ['.content > a', 'href'],
                ])
                ->range('.notice-content .notice-list:first > .content-list')
                ->query()
                ->queryData();
}

/**
 * 获取详情页内容
 * 
 * @param string  $url  链接
 * @param string  $find  选择器
 * @return string
 */
function get_context(string $url, string $find) {
    $context =  QueryList::get($url)
                    ->find($find);

    return normalize_context($context);
}

/**
 * 保存推送
 * 
 * @param string  $provider  厂商
 * @param string  $provider_name  厂商名
 * @param string  $find  选择器
 * @param array  $data  数据
 * @return void
 */
function save_push(string $provider, string $provider_name, string $find, array $data) {
    global $config, $push;

    $provider_data = $config[$provider];

    foreach($data as $item) {
        $push[] = [$provider_name, $item['name'], get_context($item['href'], $find . ' > *')];

        array_unshift($provider_data, $item);
    }

    $config[$provider] = array_slice($provider_data, 0, 30);

    return;
}

/**
 * 推送
 * wiki: http://sc.ftqq.com/?c=code
 *
 * @param string  $provider  厂商
 * @param string  $name  名称
 * @param string  $context  内容
 * @return string|bool
 */
function send($provider, $name, $context) {
    $api_url = 'https://sc.ftqq.com/' . getenv('sckey') . '.send';

    return  file_get_contents($api_url, false, stream_context_create([
                'http' => [
                    'method'    => 'POST',
                    'header'    => 'Content-type: application/x-www-form-urlencoded',
                    'content'   => sprintf('text=%s-%s&desp=%s', $provider, $name, $context),
                ],
            ]));
}

/**
 * 格式化列表
 * 将列表的 name 设置为 key
 * 
 * @param array  $data  数据
 * @return array
 */
function normalize_list(array $data) {
    return  collect($data)
                ->keyBy('name')
                ->all();
}

/**
 * 格式化内容
 *
 * @param \QL\QueryList  $ql  ql
 * @return string
 */
function normalize_context($ql) {
    return  $ql->htmls()
                ->map(function($item) {
                    return urlencode(str_replace('<br>', urldecode('%0D%0A%0D%0A'), $item));
                })
                // 换行符号
                ->implode('%0D%0A%0D%0A');
}

/**
 * 获取配置
 *
 * @return array
 */
function get_config() {
    global $oss, $bucket, $config_name;

    $exist = $oss->doesObjectExist($bucket, $config_name);

    if(!$exist) {
        return [];
    }

    $config = $oss->getObject($bucket, $config_name);

    return json_decode($config, true);
}

/**
 * 保存配置
 * 
 * @param array  $config  配置
 * @return void
 */
function save_config(array $config) {
    global $oss, $bucket, $config_name;

    return $oss->putObject($bucket, $config_name, json_encode($config, JSON_UNESCAPED_UNICODE));
}

?>